import 'dart:async';
import 'dart:convert';
import 'dart:math';

import 'package:the_paint/src/generated/client.pb.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import 'package:web_socket_channel/web_socket_channel.dart';

class WebRTCChannels extends Resolver {
  WebRTCChannels._internal();

  static final WebRTCChannels _singleton = WebRTCChannels._internal();
  static final Map<String, Duplex<RTCDataChannelMessage>> mapf = {};

  // RTCPeerConnection? _pc;
  static bool connected = false;

  Stream<RTCData>? _rtcData;

  factory WebRTCChannels() {
    // _singleton.connect(() {});
    return _singleton;
  }

  final Map<String, dynamic> _iceServers = {
    'iceServers': [
      {
        'url': 'stun:melvans.com:3478',
        'username': 'b84e4232f84505e3c3967184df374f2cdaa954f1',
        'credential': 'afc39ae0591c132292f73a87f6f3725311fbafa0'
        // 'url': 'stun:stun.l.google.com:19302'
      },
    ]
  };

  final Map<String, dynamic> _config = {
    'mandatory': {},
    'optional': [
      {'DtlsSrtpKeyAgreement': true},
    ],
  };

  final Map<String, dynamic> _constraints = {
    'mandatory': {},
    'optional': [],
  };

  final _channel = WebSocketChannel.connect(
    Uri.parse('ws://localhost:8081/ws'),
  );

  Future<void> connect() async {
    if (connected) {
      return;
    }
    connected = true;
    print("Connecting WEBSOCKET...");

    _rtcData = _channel.stream
        .map((event) => RTCData.fromBuffer(event))
        .asBroadcastStream();
  }

  Future<RTCData> waitForData(String event /*add client id*/) async {
    await connect();
    return await _rtcData!.firstWhere((msg) => msg.event == event);
  }

  Future<Duplex<RTCDataChannelMessage>> createAndSubscribePeer(
      RTCSessionDescription session) async {
    Duplex<RTCDataChannelMessage> dup = Duplex();
    var key = session.type!;
    if (mapf.containsKey(key)) {
      return mapf[key]!;
    }
    mapf[key] = dup;

    var pc = await createPeerConnection(
      _iceServers,
    );

    await pc.setRemoteDescription(session);
    var answer = await pc.createAnswer();
    await pc.setLocalDescription(answer);
    await sendLocalDescription(pc);

    pc.onDataChannel = (channel) {
      print('\nnew channel ${channel.id} ${channel.label}');
      channel.onDataChannelState = (state) {
        print('\ndc2: change state: ${state.toString()}');
      };
      channel.onMessage = (data) {
        print(data.binary.length);
        dup.streamToUI.add(data);
      };

      if (channel.id == 1) {
        dup.streamFromUI.stream.forEach((element) {
          channel.send(element);
        });
      }
    };
    return dup;
  }

  sendLocalDescription(RTCPeerConnection pc) async {
    RTCSessionDescription? answer = await pc.getLocalDescription();
    if (answer == null) {
      return;
    }
    final encodedData =
        RTCData(data: answer.sdp, event: answer.type).writeToBuffer();
    _channel.sink.add(encodedData);
  }

  @override
  Future<Duplex<RTCDataChannelMessage>> dataChannel(Point<int> point) async {
    final encodedData =
        RTCData(data: point.toString(), event: "req_p").writeToBuffer();
    _channel.sink.add(encodedData);
    var msg = await waitForData("offer");
    var session = RTCSessionDescription(msg.data, msg.event);
    return await createAndSubscribePeer(session);
  }
}

class Duplex<T> {
  StreamController<T> streamToUI = StreamController.broadcast();
  StreamController<T> streamFromUI = StreamController.broadcast();

  void send(T serialize) {
    streamFromUI.sink.add(serialize);
  }

  Stream<T> getStream() {
    return streamToUI.stream;
  }
}

/* Return to who we need to connect*/
abstract class Resolver {
  Future<Duplex<RTCDataChannelMessage>> dataChannel(Point<int> point);
}
