import 'dart:async';
import 'dart:math';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import 'package:the_paint/src/generated/client.pb.dart';
import 'package:the_paint/src/square/square.dart';

import '../connection/ws.dart';

class WebRTCSquareData extends SquereData {
  final Resolver resolver = WebRTCChannels();
  final StreamController<MonitorReply> streamChannel =
      StreamController<MonitorReply>();

  Duplex? _dc;

  WebRTCSquareData(super.tile);

  @override
  Stream<MonitorReply> getSteam() {
    resolver.dataChannel(super.tile).then((dc) {
      _dc = dc;
      streamChannel.addStream(dc.getStream().map((event) {
        return MonitorReply.fromBuffer(event.binary);
      }).where((event) => event.tile.x == tile.x && event.tile.y == tile.y));

      // request
      var req = WebRTCRequest(
        requestData: RequestData(tile: BPoint(x: tile.x, y: tile.y)),
      );

      _dc!.send(
        RTCDataChannelMessage.fromBinary(
          req.writeToBuffer(),
        ),
      );
    });
    return streamChannel.stream;
  }

  @override
  void paint(Point<int> point, Color color) {
    if (_dc == null) {
      return;
    }
    // request
    var req = WebRTCRequest(
      requestData: RequestData(tile: BPoint(x: tile.x, y: tile.y)),
    );

    _dc!.send(
      RTCDataChannelMessage.fromBinary(
        req.writeToBuffer(),
      ),
    );

    // _dc!.send(serialize(point, color));
  }

  RTCDataChannelMessage serialize(Point<int> point, Color color) {
    final data = MonitorPoint(
      point: BPoint(x: point.x, y: point.y),
      color: fromColor(color),
    ).writeToBuffer();
    return RTCDataChannelMessage.fromBinary(data);
  }

  BColor fromColor(Color c) {
    return BColor(rgba: c.red | c.green << 8 | c.blue << 16 | c.alpha << 24);
  }
}
